SBLOCK = 10
XMAX = 80
YMAX = 60
FACTOR = 1.2

function newstate()
	local t = {}
	for i=1,XMAX do
		t[i] = {}
		for j=1,YMAX do
			t[i][j] = {math.random(0, 255), math.random(0, 255), math.random(0, 255)}
		end
	end
	return t
end

function love.load()
	s1 = newstate()
	s2 = newstate()
	time = 0
end

function avg(s1, i, j, k)
	local t = s1[i][j][k]
	local n = 1
	if s1[i-1] then
		t = t + s1[i-1][j][k]
		n = n + 1
		if s1[i-1][j-1] then
			t = t + s1[i-1][j-1][k]
			n = n + 1
		end
		if s1[i-1][j+1] then
			t = t + s1[i-1][j+1][k]
			n = n + 1
		end
	end
	if s1[i+1] then
		t = t + s1[i+1][j][k]
		n = n + 1
		if s1[i+1][j-1] then
			t = t + s1[i+1][j-1][k]
			n = n + 1
		end
		if s1[i+1][j+1] then
			t = t + s1[i+1][j+1][k]
			n = n + 1
		end
	end
	if s1[i][j-1] then
		t = t + s1[i][j-1][k]
		n = n + 1
	end
	if s1[i][j+1] then
		t = t + s1[i][j+1][k]
		n = n + 1
	end
	return t/n
end

function evolve(s1, i, j, k)
	local a = avg(s1, i, j, k)
	local b = avg(s1, i, j, (k) % 3 + 1)
	local c = avg(s1, i, j, (k - 2) % 3 + 1)
	return math.min(255, math.max(0, a + FACTOR*(256 - b - c)))
end

function step()
	for i=1,XMAX do
		for j=1,YMAX do
			for k=1,3 do
				s2[i][j][k] = evolve(s1, i, j, k)
			end
		end
	end
	s1, s2 = s2, s1
end

function love.update(dt)
	time = time + dt
	if time > 0.1 then
		step()
		if love.mouse.isDown('l') then
			local x, y = love.mouse.getPosition()
			local i = 1 + math.floor(x / SBLOCK)
			local j = 1 + math.floor(y / SBLOCK)
			s1[i][j] = {math.random(0, 255), math.random(0, 255), math.random(0, 255)}
		elseif love.mouse.isDown('r') then
			local x, y = love.mouse.getPosition()
			local i = 1 + math.floor(x / SBLOCK)
			local j = 1 + math.floor(y / SBLOCK)
			for ii = i - 1, i + 1 do
				if s1[ii] then
					for jj = j - 1, j + 1 do
						if s1[ii][jj] then
							for kk = 1, 3 do
								s1[ii][jj][kk] = 255 - s1[ii][jj][kk]
							end
						end
					end
				end
			end
		end
		time = time - 0.1
	end
end

function love.draw()
	for i=1,XMAX do
		for j=1,YMAX do
			love.graphics.setColor(s1[i][j])
			love.graphics.rectangle('fill', i * SBLOCK - SBLOCK, j * SBLOCK - SBLOCK, SBLOCK, SBLOCK)
		end
	end
	love.graphics.setColor(0, 0, 0)
	love.graphics.print("Reset with r, close with escape", 5, 5)
end

function love.keypressed(key)
	if key == 'escape' then
		love.event.quit()
	elseif key == 'r' then
		s1 = newstate()
	end
end
